/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_alg.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_alg.h"
//---------------------------------------------------------------------------

const UINT g_sel_NSGA = 0;
const UINT g_sel_IBEA = 1;
const UINT g_sel_SPEA = 2;
//---------------------------------------------------------------------------

DemoAlg::DemoAlg(string ini_f_name, string ini_args)
{
	ifstream f_ini;
	f_ini.open(ini_f_name.c_str(), fstream::in);

	try {
		if (!f_ini.good()) {
			throw E("Could not open the file " + ini_f_name);
		}
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	}

	m_params = new DemoParams();
	m_files = new FileStat();

	m_probl_fun = new ProblFun();
	m_probl_sim = new ProblSim();

	m_pop_nsga = new DemoPopNsga();
	m_pop_ibea = new DemoPopIbea();
	m_pop_spea = new DemoPopSpea();

	m_seed = 0;
	m_sel_type = g_sel_NSGA;
	m_always_add = false;

	ReadFromFile(ini_f_name);
	if (ini_args != "") {
		ReadFromString(ini_args);
	}

	InitAlg();

	f_ini.close();
}
//---------------------------------------------------------------------------

DemoAlg::~DemoAlg()
{
	delete m_params;
	delete m_files;
	delete m_probl_fun;
	delete m_probl_sim;
	delete m_pop_nsga;
	delete m_pop_ibea;
	delete m_pop_spea;
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromFile(string f_name)
{
	ifstream f;
	f.open(f_name.c_str(), fstream::in);

	ReadFromStream(f);
	f.close();
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromString(string args)
{
	istringstream str(args.c_str(), stringstream::in);

	ReadFromStream(str);
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromStream(istream &stream)
{
	string tmp_str;
	UINT tmp_int;
	double tmp_double;
	bool tmp_bool;

	try {
		while (!stream.eof()) {
			stream >> tmp_str;

			if (stream.eof())
				break;

			if (tmp_str == "%") {
				stream.ignore(256, '\n');
			}

			else if (tmp_str == "seed") {
				stream >> m_seed;
			}
			else if (tmp_str == "pop_size") {
				stream >> tmp_int;
				if (tmp_int < 5)
					throw E("Error in parameter \"pop_size\" (it should be greater than 4)");
				m_params->SetPopSize(tmp_int);
			}
			else if (tmp_str == "max_eval") {
				stream >> tmp_int;
				if (tmp_int < 5)
					throw E("Error in parameter \"max_eval\" (it should be greater than 4)");
				m_params->SetMaxEval(tmp_int);
			}
			else if (tmp_str == "weight") {
				stream >> tmp_double;
				if ((tmp_double < 0) || (tmp_double > 2))
					throw E("Error in parameter \"weight\" (it should lie in interval [0, 2])");
				m_params->SetWeight(tmp_double);
			}
			else if (tmp_str == "cross_prob") {
				stream >> tmp_double;
				if ((tmp_double < 0) || (tmp_double > 1))
					throw E("Error in parameter \"cross_prob\" (it should lie in interval [0, 1])");
				m_params->SetCrossProb(tmp_double);
			}
			else if (tmp_str == "sel_type") {
				stream >> m_sel_type;
				if ((m_sel_type != g_sel_NSGA) && (m_sel_type != g_sel_IBEA)
					&& (m_sel_type != g_sel_SPEA))
					throw E("Error in parameter \"sel_type\"");
			}
			else if (tmp_str == "always_add") {
				stream >> m_always_add;
			}

			else if (tmp_str == "elitism") {
				stream >> tmp_bool;
				m_pop_nsga->SetElitism(tmp_bool);
			}
			else if (tmp_str == "elitism_r") {
				stream >> tmp_double;
				if ((tmp_double <= 0) || (tmp_double > 1))
					throw E("Error in parameter \"elitism_r\" (it should lie in interval (0, 1]");
				m_pop_nsga->SetElitismR(tmp_double);
			}
			else if (tmp_str == "indicator") {
				stream >> tmp_int;
				if ((tmp_int != g_ibea_eps) && (tmp_int != g_ibea_hyp))
					throw E("Error in parameter \"indicator\"");
				m_pop_ibea->SetIndicator(tmp_int);
			}
			else if (tmp_str == "rho") {
				stream >> tmp_double;
				if (tmp_double <= 1)
					throw E("Error in parameter \"rho\" (it should be greater than 1)");
				m_pop_ibea->SetRho(tmp_double);
			}
			else if (tmp_str == "kappa") {
				stream >> tmp_double;
				if ((tmp_double <= 0) || (tmp_double > 1))
					throw E("Error in parameter \"kappa\" (it should lie in interval (0, 1])");
				m_pop_ibea->SetKappa(tmp_double);
			}

			else if (tmp_str == "func_type") {
				stream >> tmp_int;
				if ((tmp_int < g_prob_min) || (tmp_int > g_prob_max))
					throw E("Error in parameter \"func_type\"");
				m_probl_sim->SetFunction(tmp_int);
				m_probl_fun->SetFunction(tmp_int);
			}
			else if (tmp_str == "num_var") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"num_var\" (it should be greater than 0)");
				m_probl_sim->SetNumVar(tmp_int);

				vector<double> var_min;
				vector<double> var_max;
				vector<double> var_disc;

				for (UINT i = 0; i < tmp_int; i++) {
					stream >> tmp_double; var_min.push_back(tmp_double);
					stream >> tmp_double; var_max.push_back(tmp_double);
					stream >> tmp_double; var_disc.push_back(tmp_double);
					if ((var_min.at(i) >= var_max.at(i)) ||
						(var_disc.at(i) > var_max.at(i) - var_min.at(i))) {
						throw E("Error in variable min/max/discretization");
					}
				}
				m_probl_sim->SetVarMin(var_min);
				m_probl_sim->SetVarMax(var_max);
				m_probl_sim->SetVarDisc(var_disc);
			}
			else if (tmp_str == "num_func") {
				stream >> tmp_int;
				if (tmp_int < 2)
					throw E("Error in parameter \"num_func\" (it should be greater than 1)");
				m_probl_sim->SetNumFunc(tmp_int);

				vector<double> func_lower;
				vector<double> func_upper;

				for (UINT i = 0; i < tmp_int; i++) {
					stream >> tmp_double; func_lower.push_back(tmp_double);
					stream >> tmp_double; func_upper.push_back(tmp_double);
					if (func_lower.at(i) >= func_upper.at(i)) {
						throw E("Error in function bounds");
					}
				}
				m_probl_sim->SetFuncLower(func_lower);
				m_probl_sim->SetFuncUpper(func_upper);
			}
			else if (tmp_str == "num_feat") {
				stream >> tmp_int;
				m_probl_sim->SetNumFeat(tmp_int);
			}

			else if (tmp_str == "command") {
				stream >> tmp_str;
				m_probl_sim->SetCommand(tmp_str);
			}
			else if (tmp_str == "archive") {
				stream >> tmp_str;
				m_probl_sim->SetArchive(tmp_str);
			}
			else if (tmp_str == "archive_read") {
				stream >> tmp_bool;
				m_probl_sim->SetArchiveRead(tmp_bool);
			}
			else if (tmp_str == "archive_write") {
				stream >> tmp_bool;
				m_probl_sim->SetArchiveWrite(tmp_bool);
			}
			else if (tmp_str == "sim_in") {
				stream >> tmp_str;
				m_probl_sim->SetSimIn(tmp_str);
			}
			else if (tmp_str == "sim_out") {
				stream >> tmp_str;
				m_probl_sim->SetSimOut(tmp_str);
			}

			else if (tmp_str == "wfg_M") {
				stream >> tmp_int;
				if (tmp_int < 2)
					throw E("Error in parameter \"wfg_M\" (it should be greater than 1)");
				else if ((tmp_int > 21) && (m_probl_fun->GetFunction() == g_prob_fun_DTLZ6s))  
					throw E("Error in parameter \"wfg_M\" or \"func_type\".");
				else if ((tmp_int > 11) && (m_probl_fun->GetFunction() == g_prob_fun_DTLZ7s))
					throw E("Error in parameter \"wfg_M\" or \"func_type\".");
				m_probl_fun->SetWfgM(tmp_int);
			}
			else if (tmp_str == "wfg_k") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"wfg_k\" (it should be greater than 0)");
				if (tmp_int % (m_probl_fun->GetWfgM() - 1) != 0)
					throw E("Error in parameter \"wfg_k\" (wfg_M - 1 should divide wfg_k)");
				m_probl_fun->SetWfgK(tmp_int);
			}
			else if (tmp_str == "wfg_l") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"wfg_l\" (it should be greater than 0)");
				m_probl_fun->SetWfgL(tmp_int);
			}
                          
			else if (tmp_str == "log_file_name") {
				stream >> tmp_str;
				m_files->GetFileLog()->SetName(tmp_str);
			}
			else if (tmp_str == "log_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"log_mode\"");
				m_files->GetFileLog()->SetMode(tmp_int);
			}        
			else if (tmp_str == "front_file_name") {
				stream >> tmp_str;
				m_files->GetFileFront()->SetName(tmp_str);
			}
			else if (tmp_str == "front_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"front_mode\"");
				m_files->GetFileFront()->SetMode(tmp_int);
			}
			else if (tmp_str == "front_gen") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"front_gen\" (it should be greater than 0)");
				m_files->SetFrontGen(tmp_int);
			}
			else if (tmp_str == "front_long") {
				stream >> tmp_bool;
				m_files->SetFrontLong(tmp_bool);
			}
			else if (tmp_str == "gen_file_name") {
				stream >> tmp_str;
				m_files->GetFileGen()->SetName(tmp_str);
			}
			else if (tmp_str == "gen_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"gen_mode\"");
				m_files->GetFileGen()->SetMode(tmp_int);
			}          
			else if (tmp_str == "gen_hyp_method") {
				stream >> tmp_int;
				if ((tmp_int != g_gen_hyp_none) && (tmp_int != g_gen_hyp_exact)
					&& (tmp_int != g_gen_hyp_estimate) && (tmp_int != g_gen_hyp_both))
					throw E("Error in parameter \"gen_hyp_method\"");
				m_files->SetGenHypMethod(tmp_int);
			}           
			else if (tmp_str == "gen_num_MC_points") {
				stream >> tmp_int;
				if (tmp_int < 100)
					throw E("Error in parameter \"gen_num_MC_points\"");
				m_files->SetGenNumMCPoints(tmp_int);
			}
			else if (tmp_str == "lines_file_name") {
				stream >> tmp_str;
				m_files->GetFileLines()->SetName(tmp_str);
			}
			else if (tmp_str == "lines_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"lines_mode\"");
				m_files->GetFileLines()->SetMode(tmp_int);
			}
			else {
				throw E("Unknown parameter: " + tmp_str);
			}
		}
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	} catch (...) {
		cerr << "Error occurred when processing parameters!" << endl;
		set_terminate(my_terminate);
		terminate();
	}
}
//---------------------------------------------------------------------------

void DemoAlg::InitAlg()
{
	if (m_seed == 0) {
		randomize();
		m_seed = random(MAXINT);
	}
	srand(m_seed);

	if (m_probl_fun->GetFunction() == g_prob_extern)
		m_probl = m_probl_sim;
	else
		m_probl = m_probl_fun;

	m_probl->InitProblem();

	m_files->InitFileStat();

	if (m_sel_type == g_sel_NSGA)
		m_pop = m_pop_nsga;
	else if (m_sel_type == g_sel_IBEA)
		m_pop = m_pop_ibea;
	else if (m_sel_type == g_sel_SPEA)
		m_pop = m_pop_spea;

	m_pop->InitPop(m_params, m_probl);

	OutputInfoGen();
	OutputInfoLog();
}
//---------------------------------------------------------------------------

void DemoAlg::Evolution()
{
	m_pop->CreateFirstPop();
	m_pop->EvaluatePop(m_files);
	m_files->OutputPop(m_pop, 1);

	vector<UINT> indexes;
	UINT domination;

	while (m_probl->GetNumEval() < m_params->GetMaxEval()) {
		indexes = m_pop->SortPopRandom();
		m_pop->TruncPop(indexes);

		for (UINT i = 0; i < m_params->GetPopSize(); i++) {
			DemoInd *candidate = new DemoInd();
			candidate->SetGenes(m_pop->GetCandidate(i));
			candidate->Evaluate(m_probl);

			domination = m_probl->GetDomination(candidate, m_pop->GetInd(i));
			m_files->OutputInd(candidate,
												 m_pop->GetInd(i),
												 m_probl->GetNumEval(),
												 domination);

			if (m_always_add) {
				m_pop->AppendInd(candidate);
			}
			else {
				if (domination == g_domin_first) {
					*m_pop->GetInd(i) = *candidate;
					delete candidate;
				}
				else if (domination == g_domin_second) {
					delete candidate;
				}
				else if (domination == g_domin_none) {
					m_pop->AppendInd(candidate);
				}
			}

		}
		indexes = m_pop->SortPop();
		m_pop->TruncPop(indexes);

		m_files->OutputPop(m_pop, (m_probl->GetNumEval() / m_params->GetPopSize()));
	}
}
//---------------------------------------------------------------------------

void DemoAlg::OutputInfoGen()
{
	if (!m_files->GetFileGen()->IsOff()) {
		stringstream tmp;
		tmp << endl << "gen_number" << "\t";
		tmp << "num_feasible" << "\t";
		tmp << "num_unfeasible" << "\t";
		tmp << "num_non_evaluated" << "\t";
		tmp << "avg_violation" << "\t";    
		tmp << "num_nondominated" << "\t";
		tmp << "exact_hyp" << "\t";
		tmp << "hyp_estimation" << "\t";
		tmp << "c_better" << "\t";
		tmp << "p_better" << "\t";
		tmp << "incomparable";
		m_files->GetFileGen()->Output(tmp.str());
	}
}
//---------------------------------------------------------------------------

void DemoAlg::OutputInfoLog()
{
	if (!m_files->GetFileLog()->IsOff()) {
		stringstream tmp;
		tmp << "Information on this run:" << endl;
		tmp << "\tseed\t" << m_seed << endl;
		tmp << endl << "Parameters of the algorithm:" << endl;
		tmp << "\tpop_size\t" << m_params->GetPopSize() << endl;
		tmp << "\tmax_eval\t" << m_params->GetMaxEval() << endl;
		tmp << "\tweight\t" << m_params->GetWeight() << endl;
		tmp << "\tcross_prob\t" << m_params->GetCrossProb() << endl;
		tmp << "\talways_add\t" << GetBoolStringYN(m_always_add) << endl;
		tmp << "\tsel_type\t";
		if (m_sel_type == g_sel_NSGA) {
			tmp << "NSGA-II" << endl;
			tmp << "\telitism\t" << GetBoolStringYN(m_pop_nsga->GetElitism()) << endl;
			tmp << "\telitism_r\t" << m_pop_nsga->GetElitismR() << endl;
		}
		else if (m_sel_type == g_sel_IBEA) {
			tmp << "IBEA" << endl;
			tmp << "\tindicator\t";
			if (m_pop_ibea->GetIndicator() == g_ibea_eps)
				tmp << "eps" << endl;
			else if (m_pop_ibea->GetIndicator() == g_ibea_hyp)
				tmp << "hyp" << endl;
			tmp << "\trho\t" << m_pop_ibea->GetRho() << endl;
			tmp << "\tkappa\t" << m_pop_ibea->GetKappa() << endl;
		}
		else if (m_sel_type == g_sel_SPEA) {
			tmp << "SPEA" << endl;
		}

		tmp << endl << "Parameters of the problem:" << endl;
		tmp << "\tfunc_type\t" << m_probl->GetProblemString() << endl;
		tmp << "\tnum_var\t" << m_probl->GetNumVar() << endl;
		tmp << m_probl->GetVariableString();
		tmp << "\tnum_func\t" << m_probl->GetNumFunc() << endl;
		tmp << m_probl->GetFunctionString();
		tmp << "\tnum_feat\t" << m_probl->GetNumFeat() << endl;
		if (m_probl->GetFunction() == g_prob_extern) {
			tmp << "\tcommand\t" << m_probl_sim->GetCommand() << endl;
			tmp << "\tarchive\t" << m_probl_sim->GetArchive() << endl;
			tmp << "\tarchive_read\t" << GetBoolStringYN(m_probl_sim->GetArchiveRead()) << endl;
			tmp << "\tarchive_write\t" << GetBoolStringYN(m_probl_sim->GetArchiveWrite()) << endl;
			tmp << "\tsim_in\t" << m_probl_sim->GetSimIn() << endl;
			tmp << "\tsim_out\t" << m_probl_sim->GetSimOut() << endl;
		}
		if ((m_probl->GetFunction() >= g_prob_fun_WFG1) &&
				(m_probl->GetFunction() <= g_prob_fun_DTLZ7s)) {
			tmp << "\twfg_M\t" << m_probl_fun->GetWfgM() << endl;
		}     
		if ((m_probl->GetFunction() >= g_prob_fun_WFG1) &&
				(m_probl->GetFunction() <= g_prob_fun_WFG9)) {
			tmp << "\twfg_k\t" << m_probl_fun->GetWfgK() << endl;
			tmp << "\twfg_l\t" << m_probl_fun->GetWfgL() << endl;
		}

		tmp << endl << "Parameters for the output:" << endl;
		tmp << "\tlog_file_name\t" << m_files->GetFileLog()->GetName() << endl;
		tmp << "\tlog_mode\t" << m_files->GetFileLog()->GetModeString() << endl;
		tmp << "\tfront_file_name\t" << m_files->GetFileFront()->GetName() << endl;
		tmp << "\tfront_mode\t" << m_files->GetFileFront()->GetModeString() << endl;
		tmp << "\tfront_gen\t" << m_files->GetFrontGen() << endl;
		tmp << "\tfront_long\t" << GetBoolStringYN(m_files->GetFrontLong()) << endl;
		tmp << "\tgen_file_name\t" << m_files->GetFileGen()->GetName() << endl;
		tmp << "\tgen_mode\t" << m_files->GetFileGen()->GetModeString() << endl;
		tmp << "\tgen_hyp_method\t" << m_files->GetGenMethodString() << endl;   
		tmp << "\tgen_num_MC_points\t" << m_files->GetGenNumMCPoints() << endl;
		tmp << "\tlines_file_name\t" << m_files->GetFileLines()->GetName() << endl;
		tmp << "\tlines_mode\t" << m_files->GetFileLines()->GetModeString() << endl;
		tmp << endl;

		m_files->GetFileLog()->Output(tmp.str());
	}
}
//---------------------------------------------------------------------------

