/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_pop.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_pop.h"
#include "demo_utilities.h"
#include "probl.h"
//---------------------------------------------------------------------------

DemoPop::DemoPop()
{
}
//---------------------------------------------------------------------------

DemoPop::~DemoPop()
{
	for (UINT i = 0; i < m_inds.size(); i++)
		delete m_inds.at(i);
}
//---------------------------------------------------------------------------

void DemoPop::InitPop(DemoParams *params, Probl *probl)
{
	m_params = params;
	m_probl = probl;

	m_inds.clear();
	for (UINT i = 0; i < m_params->GetPopSize(); i++) {
		DemoInd *ind = new DemoInd();
		m_inds.push_back(ind);
	}
}
//---------------------------------------------------------------------------

void DemoPop::CreateFirstPop()
{
	for (UINT i = 0; i < m_params->GetPopSize(); i++)
		m_inds.at(i)->SetGenes(m_probl->GetRandVector());
}
//---------------------------------------------------------------------------

void DemoPop::EvaluatePop(FileStat *files)
{
	for (UINT i = 0; i < m_inds.size(); i++) {
		m_inds.at(i)->Evaluate(m_probl);
		files->OutputInd(m_inds.at(i), NULL, m_probl->GetNumEval(), g_domin_original);
	}
}
//---------------------------------------------------------------------------

vector<double> DemoPop::GetCandidate(UINT parent)
{
	UINT i1, i2, i3;
	UINT i, i_rand;
	vector<double> result;
	result.clear();
	for (i = 0; i < m_probl->GetNumVar(); i++)
		result.push_back(0);

	do i1 = random(m_inds.size());
	while (i1 == parent);

	do i2 = random(m_inds.size());
	while ((i2 == parent) || (i2 == i1));

	do i3 = random(m_inds.size());
	while ((i3 == parent) || (i3 == i1) || (i3 == i2));

	i_rand = random(m_probl->GetNumVar());
	i = i_rand;
	for (UINT k = 0; k < m_probl->GetNumVar(); k++) {
		if (random(100) < (100 * m_params->GetCrossProb()) || (i == i_rand))
			result.at(i) =
				m_params->GetWeight() *
				(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i)) +
				m_inds.at(i3)->GetGene(i);
		else
			result.at(i) = m_inds.at(parent)->GetGene(i);
		i = ++i % m_probl->GetNumVar();
	}
	return m_probl->GetRoundVector(result);
}
//---------------------------------------------------------------------------

vector<UINT> DemoPop::SortPop()
{
	vector<UINT> result;
	result.clear();
	return result;
}
//---------------------------------------------------------------------------

vector<UINT> DemoPop::SortPopRandom()
{
	UINT i;
	vector<double> rand_num;
	rand_num.clear();

	for (i = 0; i < m_inds.size(); i++)
		rand_num.push_back((double) rand() / RAND_MAX);
	return GetSorted(rand_num);
}
//---------------------------------------------------------------------------

void DemoPop::TruncPop(vector<UINT> &sort_vect)
{
	UINT i;
	UINT size = sort_vect.size();
	vector<DemoInd*> inds;
	inds.clear();

	for (i = 0; i < size; i++) {
		DemoInd *ind = new DemoInd();
		*ind = *m_inds.at(sort_vect.at(i));
		inds.push_back(ind);
	}

	for (i = 0; i < size; i++)
		*m_inds.at(i) = *inds.at(i);
	while (m_inds.size() > size) {
		delete m_inds.at(m_inds.size() - 1);
		m_inds.pop_back();
	}
	for (i = 0; i < size; i++)
		delete inds.at(i);
}
//---------------------------------------------------------------------------

void DemoPop::AppendInd(DemoInd *ind)
{
	m_inds.push_back(ind);
}
//---------------------------------------------------------------------------

string DemoPop::GetPopString()
{
	UINT i;
	stringstream result;

	for (i = 0; i < m_inds.size(); i++)
		result << m_inds.at(i)->GetIndString();

	return result.str();
}
//---------------------------------------------------------------------------

string DemoPop::GetPopCritString()
{
	UINT i;
	stringstream result;

	for (i = 0; i < m_inds.size(); i++)
		result << m_inds.at(i)->GetIndCritString();

	return result.str();
}
//---------------------------------------------------------------------------

string DemoPop::GetFrontString(bool front_long)
{
	stringstream result;
	UINT i, j, dominate;

	vector<UINT> n;
	n.clear();

	for (i = 0; i < m_inds.size(); i++) {
		n.push_back(0);
		for (j = 0; j < m_inds.size(); j++) {
			if (i == j) continue;
			dominate = m_probl->GetDomination(m_inds.at(i), m_inds.at(j));
			if (dominate == g_domin_second)
				n.at(i)++;
		}
		if (n.at(i) == 0) {
			if (front_long)
				result << m_inds.at(i)->GetIndString();
			else
				result << m_inds.at(i)->GetIndCritString();
		}
	}

	return result.str();
}
//---------------------------------------------------------------------------

