/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_pop_nsga.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_pop_nsga.h"
#include <values>
//---------------------------------------------------------------------------

DemoPopNsga::DemoPopNsga()
	: DemoPop()
{
	m_elitism = false;
	m_elitism_r = 0.5;

	m_crow_dist.clear();
	m_ranks.clear();
}
//---------------------------------------------------------------------------

DemoPopNsga::~DemoPopNsga()
{
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopNsga::SortPop()
{
	if (m_inds.size() == m_params->GetPopSize()) {
		vector<UINT> result;
		result.clear();
		for (UINT i = 0; i < m_inds.size(); i++)
			result.push_back(i);
		return result;
	}

	CalcFronts();
	if (m_elitism == false)
		return SortPopNsga();
	else
		return SortPopNsgaElitism();
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopNsga::SortPopNsga()
{
	vector<UINT> result;
	result.clear();

	UINT i, j;
	bool exact = false;
	UINT num_ind = 0;
	UINT count_entire = 0;

	do {
		num_ind += m_fronts.at(count_entire++).size();
		if (num_ind == m_params->GetPopSize())
			exact = true;
	}
	while (num_ind < m_params->GetPopSize());
	if (!exact)
		count_entire--;

	for (i = 0; i < count_entire; i++)
		for (j = 0; j < m_fronts.at(i).size(); j++)
			result.push_back(m_fronts.at(i).at(j));

	if (!exact) {
		CalcCrowDist(count_entire);
		vector<double> crowding;
		crowding.clear();
		for (i = 0; i < m_fronts.at(count_entire).size(); i++) {
			crowding.push_back(-(double)m_crow_dist.at(m_fronts.at(count_entire).at(i)));
		}
		vector<UINT> indexes = GetSorted(crowding);

		i = 0;
		do {
			result.push_back(m_fronts.at(count_entire).at(indexes.at(i++)));
		}
		while (result.size() < m_params->GetPopSize());
	}

	return result;
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopNsga::SortPopNsgaElitism()
{
	vector<UINT> result;
	result.clear();

	UINT i, j;
	UINT rest, used, num_fronts, num_available;
	num_fronts = m_fronts.size();

	vector<UINT> num_used_from_front;
	num_used_from_front.clear();
	for (i = 0; i < num_fronts; i++)
		num_used_from_front.push_back(0);

	rest = 0;
	used = 0;

	while (used < m_params->GetPopSize()) {
		for (i = 0; i < num_fronts; i++) {
			num_available = ceil(m_inds.size() * (1 - m_elitism_r) *
				pow(m_elitism_r, i) / (1 - pow(m_elitism_r, num_fronts))) + rest;
			if (m_fronts.at(i).size() == num_used_from_front.at(i)) continue;
			if (m_fronts.at(i).size() <= num_available) {
				for (j = 0; j < m_fronts.at(i).size(); j++)
					result.push_back(m_fronts.at(i).at(num_used_from_front.at(i) + j));
				for (j = 0; j < m_fronts.at(i).size(); j++)
					num_used_from_front.at(i)++;
				rest = num_available - m_fronts.at(i).size();
			}
			else {
				CalcCrowDist(i);
				vector<double> crowding;
				crowding.clear();
				for (j = 0; j < m_fronts.at(i).size(); j++)
					crowding.push_back(-m_crow_dist.at(m_fronts.at(i).at(j)));
				vector<UINT> indexes = GetSorted(crowding);

				for (j = 0; j < num_available; j++)
					result.push_back(m_fronts.at(i).at(num_used_from_front.at(i) + indexes.at(j)));
				for (j = 0; j < num_available; j++)
					num_used_from_front.at(i)++;
				rest = 0;
			}
			if (used >= m_params->GetPopSize()) break;
		}
	}

	return result;
}
//---------------------------------------------------------------------------

void DemoPopNsga::CalcFronts()
{
	UINT i, j, k, l, dominate;
	vector< vector<UINT> > set;
	set.clear();
	vector<UINT> n;
	n.clear();
	m_fronts.clear();
	vector<UINT> tmp_set;
	vector<UINT> tmp_front;

	tmp_front.clear();
	for (i = 0; i < m_inds.size(); i++) {
		tmp_set.clear();
		n.push_back(0);
		m_ranks.push_back(0);
		for (j = 0; j < m_inds.size(); j++) {
			if (i == j)
				continue;
			dominate = m_probl->GetDomination(m_inds.at(i), m_inds.at(j));
			if (dominate == g_domin_first)
				tmp_set.push_back(j);
			else if (dominate == g_domin_second)
				n.at(i)++;
		}
		set.push_back(tmp_set);
		if (n.at(i) == 0) {
			m_ranks.at(i) = 0;
			tmp_front.push_back(i);
		}
	}
	m_fronts.push_back(tmp_front);

	k = 0;
	while (!m_fronts.at(k).empty()) {
		tmp_front.clear();
		for (i = 0; i < m_fronts.at(k).size(); i++)
			for (j = 0; j < set.at(m_fronts.at(k).at(i)).size(); j++) {
				n.at(set.at(m_fronts.at(k).at(i)).at(j))--;
				if (n.at(set.at(m_fronts.at(k).at(i)).at(j)) == 0)
					tmp_front.push_back(set.at(m_fronts.at(k).at(i)).at(j));
			}
		k++;
		if (k == m_inds.size()) break;
		m_fronts.push_back(tmp_front);
		for (l = 0; l < tmp_front.size(); l++)
			m_ranks.at(tmp_front.at(l)) = k;
	}
}
//---------------------------------------------------------------------------

void DemoPopNsga::CalcCrowDist(UINT front_num)
{
	UINT i, j, k;
	vector<UINT> front = m_fronts.at(front_num);

	m_crow_dist.clear();
	for (i = 0; i < m_inds.size(); i++)
		m_crow_dist.push_back(0);

	vector<double> sort_vec;
	vector<UINT> indexes;
	for (j = 0; j < m_probl->GetNumFunc(); j++) {
		sort_vec.clear();
		for (i = 0; i < front.size(); i++)
			sort_vec.push_back(m_inds.at(front.at(i))->GetCriterion(j));
		indexes = GetSorted(sort_vec);
		m_crow_dist.at(front.at(indexes.at(0))) += MAXINT;
		m_crow_dist.at(front.at(indexes.at(indexes.size() - 1))) += MAXINT;

		if (fabs(
				m_inds.at(front.at(indexes.at(0)))->GetCriterion(j) -
				m_inds.at(front.at(indexes.at(indexes.size() - 1)))->GetCriterion(j)
				) < g_eq_prec)
			{
				for (i = 1; i < indexes.size() - 1; i++)
					m_crow_dist.at(front.at(indexes.at(i))) += MAXINT;
			}

		else
			for (i = 1; i < indexes.size() - 1; i++) {
				m_crow_dist.at(front.at(indexes.at(i))) +=
					(m_inds.at(front.at(indexes.at(i + 1)))->GetCriterion(j) -
					 m_inds.at(front.at(indexes.at(i - 1)))->GetCriterion(j)) /
					(m_inds.at(front.at(indexes.at(indexes.size() - 1)))->GetCriterion(j) -
					 m_inds.at(front.at(indexes.at(0)))->GetCriterion(j));
			}
	}

	// Restore the crowding distance values for all copies:
	double copy_sum;
	vector<UINT> tmp_copy;
	vector< vector<UINT> > copies;

	copies.clear();
	for (i = 0; i < front.size(); i++) {
		tmp_copy.clear();
		for (j = 0; j < front.size(); j++) {
			if ((i == j) || (m_probl->IsEqualCrit(m_inds.at(front.at(i))->GetCriteria(),
											 m_inds.at(front.at(j))->GetCriteria()))) {
				tmp_copy.push_back(j);
			}
		}
		copies.push_back(tmp_copy);
	}

	for (i = 0; i < front.size(); i++) {
		copy_sum = 0;
		k = copies.at(i).size();
		if (k > 1) {

			for (j = 0; j < k; j++) {
				copy_sum += m_crow_dist.at(front.at(copies.at(i).at(j)));
			}

			m_crow_dist.at(front.at(copies.at(i).at(0))) = copy_sum;
			for (j = 1; j < k; j++) {
				m_crow_dist.at(front.at(copies.at(i).at(j))) = 0;
			}

			for (j = 1; j < k; j++) {
				tmp_copy.clear();
				tmp_copy.push_back(copies.at(i).at(j));
				copies.at(copies.at(i).at(j)) = tmp_copy;
			}
			tmp_copy.clear();
			tmp_copy.push_back(i);
			copies.at(i) = tmp_copy;
		}
	}
}
//---------------------------------------------------------------------------


