/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: file_stat.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 20.02.2006
	last change: $date$

	===========================================================================
*/

#include "file_stat.h"
//---------------------------------------------------------------------------   

const UINT g_gen_hyp_none = 0;
const UINT g_gen_hyp_exact = 1;
const UINT g_gen_hyp_estimate = 2;
const UINT g_gen_hyp_both = 3;
//---------------------------------------------------------------------------

FileStat::FileStat()
{
	m_gen = new File();
	m_front = new File();
	m_log = new File();
	m_lines = new File();

	m_front_gen = 1;
	m_front_long = false;

	m_repl_first = 0;
	m_repl_second = 0;
	m_repl_none = 0;
}
//---------------------------------------------------------------------------

FileStat::~FileStat()
{
	delete m_gen;
	delete m_front;
	delete m_log;
	delete m_lines;
}
//---------------------------------------------------------------------------

void FileStat::InitFileStat()
{
	m_gen->InitFile();
	m_front->InitFile();
	m_log->InitFile();
	m_lines->InitFile();
}
//---------------------------------------------------------------------------

void FileStat::OutputInd(DemoInd *ind,
												 DemoInd *parent,
												 UINT index,
												 UINT domination)
{
	if (!m_log->IsOff()) {
		stringstream tmp1;
		tmp1 << index << "\t" << ind->GetIndString();
		m_log->Output(tmp1.str());
	}

	if ((domination == g_domin_first) && (!m_lines->IsOff())) {
		stringstream tmp2;
		tmp2 << index << "\t" << ind->GetIndCritString();
		tmp2 << index << "\t" << parent->GetIndCritString();
		tmp2 << endl;
		m_lines->Output(tmp2.str());
	}

	if (!m_gen->IsOff()) {
		if (domination == g_domin_first)
			m_repl_first++;
		else if (domination == g_domin_second)
			m_repl_second++;
		else
			m_repl_none++;
	}
}
//---------------------------------------------------------------------------

void FileStat::OutputPop(DemoPop *pop, UINT gen)
{
	if ((gen % m_front_gen == 0) && (!m_front->IsOff())) {
		stringstream tmp1;
		tmp1 << pop->GetFrontString(m_front_long) << endl;
		m_front->Output(tmp1.str());
	}

	if (!m_gen->IsOff()) {
		UINT num_feasible = 0;
		UINT num_unfeasible = 0;
		UINT num_non_evaluated = 0;
		double avg_violation = 0;
		for (UINT i = 0; i < pop->GetInds().size(); i++) {
			if (pop->GetInd(i)->GetEvaluated() == false) {
				num_non_evaluated++;
			}
			else {
				if (pop->GetInd(i)->GetViolation() == 0) {
					num_feasible++;
				}
				else {
					avg_violation += pop->GetInd(i)->GetViolation();
					num_unfeasible++;
				}
			}
		}
		if (num_unfeasible != 0) {
			avg_violation = avg_violation / num_unfeasible;
		}

		stringstream tmp2;
		if (gen == 1) tmp2 << endl;
		tmp2.precision(g_out_prec);
		tmp2 << scientific << gen << "\t";
		tmp2 << scientific << num_feasible << "\t";
		tmp2 << scientific << num_unfeasible << "\t";
		tmp2 << scientific << num_non_evaluated << "\t";
		tmp2 << scientific << avg_violation << "\t";

		HsoPop *hso_pop = new HsoPop(pop);            
		tmp2 << scientific << hso_pop->GetIndsSize() << "\t";   
		const double zero = 0;
		switch (m_gen_hyp_method) {
			case g_gen_hyp_none:
				tmp2 << scientific << zero << "\t";
				tmp2 << scientific << zero << "\t";
				break;
			case g_gen_hyp_exact:
				tmp2 << scientific << hso_pop->GetExactHypervolume() << "\t";
				tmp2 << scientific << zero << "\t";
				break;
			case g_gen_hyp_estimate:
				tmp2 << scientific << zero << "\t";
				tmp2 << scientific << hso_pop->GetHypervolumeEstimation(m_gen_num_MC_points) << "\t";
				break;
			case g_gen_hyp_both:
				tmp2 << scientific << hso_pop->GetExactHypervolume() << "\t";
				tmp2 << scientific << hso_pop->GetHypervolumeEstimation(m_gen_num_MC_points) << "\t";
				break;
		}
		delete hso_pop;

		tmp2 << scientific << m_repl_first << "\t";
		tmp2 << scientific << m_repl_second << "\t";
		tmp2 << scientific << m_repl_none << endl;
		m_gen->Output(tmp2.str());

		m_repl_first = 0;
		m_repl_second = 0;
		m_repl_none = 0;
	}
}
//---------------------------------------------------------------------------     

string FileStat::GetGenMethodString()
{
	string result;

	switch (m_gen_hyp_method) {
		case g_gen_hyp_none: result = "not calculated"; break;
		case g_gen_hyp_exact: result = "calculated exactly"; break;
		case g_gen_hyp_estimate: result = "estimated"; break;
		case g_gen_hyp_both: result = "calculated exactly and estimated"; break;
	}
	return result;
}
//---------------------------------------------------------------------------
