/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: probl.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "probl.h"
//---------------------------------------------------------------------------

const double g_eq_prec = 0.00000001;
const UINT g_out_prec = 10;
//---------------------------------------------------------------------------

const UINT g_domin_none = 0;
const UINT g_domin_first = 1;
const UINT g_domin_second = 2;
const UINT g_domin_original = 3;
//---------------------------------------------------------------------------

const UINT g_prob_extern = 0;
const UINT g_prob_fun_ZDT1 = 1;
const UINT g_prob_fun_ZDT2 = 2;
const UINT g_prob_fun_ZDT3 = 3;
const UINT g_prob_fun_ZDT4 = 4;
const UINT g_prob_fun_ZDT6 = 5;
const UINT g_prob_fun_WFG1 = 6;
const UINT g_prob_fun_WFG2 = 7;
const UINT g_prob_fun_WFG3 = 8;
const UINT g_prob_fun_WFG4 = 9;
const UINT g_prob_fun_WFG5 = 10;
const UINT g_prob_fun_WFG6 = 11;
const UINT g_prob_fun_WFG7 = 12;
const UINT g_prob_fun_WFG8 = 13;
const UINT g_prob_fun_WFG9 = 14;
const UINT g_prob_fun_DTLZ1 = 15;
const UINT g_prob_fun_DTLZ2 = 16;
const UINT g_prob_fun_DTLZ3 = 17;
const UINT g_prob_fun_DTLZ4 = 18;
const UINT g_prob_fun_DTLZ5 = 19;
const UINT g_prob_fun_DTLZ6 = 20;
const UINT g_prob_fun_DTLZ7 = 21; 
const UINT g_prob_fun_DTLZ6s = 22;
const UINT g_prob_fun_DTLZ7s = 23;
const UINT g_prob_min = 0;
const UINT g_prob_max = 23;
//---------------------------------------------------------------------------

void Probl::SetVarMin(vector<double> var_min)
	{m_var_min = var_min;}

void Probl::SetVarMax(vector<double> var_max)
	{m_var_max = var_max;}

void Probl::SetVarDisc(vector<double> var_disc)
	{m_var_disc = var_disc;}

void Probl::SetFuncLower(vector<double> func_lower)
	{m_func_lower = func_lower;}

void Probl::SetFuncUpper(vector<double> func_upper)
	{m_func_upper = func_upper;}
//---------------------------------------------------------------------------

Probl::Probl()
{
	m_num_eval = 0;
	m_function = g_prob_fun_ZDT1;
	m_num_var = 30;
	m_num_func = 2;
	m_num_feat = 0;

	m_var_min.clear();
	m_var_max.clear();
	m_var_disc.clear();

	m_func_lower.clear();
	m_func_upper.clear();
}
//---------------------------------------------------------------------------

vector<double> Probl::GetRandVector()
{
	vector<double> result;
	result.clear();

	for (UINT i = 0; i < m_num_var; i++) {
		if (m_var_disc.at(i) == 0)
			result.push_back(m_var_min.at(i) + (m_var_max.at(i) - m_var_min.at(i)) *
				(double) rand() / RAND_MAX);
		else
			result.push_back(m_var_min.at(i) + m_var_disc.at(i) *
				random((UINT) (1 + (m_var_max.at(i) - m_var_min.at(i)) / m_var_disc.at(i))));
	}
	return result;
}
//---------------------------------------------------------------------------

vector<double> Probl::GetRoundVector(vector<double> &genes)
{
	vector<double> result;
	result.clear();

	for (UINT i = 0; i < m_num_var; i++) {
		if (genes.at(i) >= m_var_max.at(i))
			result.push_back(m_var_max.at(i));

		else if (genes.at(i) <= m_var_min.at(i))
			result.push_back(m_var_min.at(i));

		else if (m_var_disc.at(i) > 0) {
			long int tmp;
			if (genes.at(i) > 0) {
				tmp = floor(genes.at(i) / m_var_disc.at(i) + 0.5);
			}
			else {
				tmp = floor(genes.at(i) / m_var_disc.at(i) - 0.5);
			}
			result.push_back(tmp * m_var_disc.at(i));
		}
		else
			result.push_back(genes.at(i));
	}
	return result;
}
//---------------------------------------------------------------------------

bool Probl::IsEqual(vector<double> &genes1,
										vector<double> &genes2)
{
	int tmp1, tmp2;

	if (genes1.size() != genes2.size())
		return false;

	for (UINT i = 0; i < genes1.size(); i++) {
		if ((m_var_disc.at(i) == 0) && (fabs(genes1.at(i) - genes2.at(i)) > g_eq_prec))
			return false;
		else if (m_var_disc.at(i) > 0 &&
			(fabs(genes1.at(i) - genes2.at(i)) > (m_var_disc.at(i) / 2.0))) {
			return false;
		}
	}

	return true;
}
//---------------------------------------------------------------------------  

bool Probl::IsEqualCrit(vector<double> &crit1,
												vector<double> &crit2)
{
	int tmp1, tmp2;

	if (crit1.size() != crit2.size())
		return false;

	for (UINT i = 0; i < crit1.size(); i++) {

		if (fabs(crit1.at(i) - crit2.at(i)) > g_eq_prec)
			return false;
	}

	return true;
}
//---------------------------------------------------------------------------

UINT Probl::GetDomination(DemoInd *first, DemoInd *second)
{
	if (first->GetViolation() > second->GetViolation()) {
		return g_domin_second;
	}
	else if (first->GetViolation() < second->GetViolation()) {
		return g_domin_first;
	}
	else if ((first->GetViolation() == 0) && (second->GetViolation() == 0)) {
		bool first_worse = false;
		bool second_worse = false;

		for (UINT i = 0; i < m_num_func; i++) {
			if (first->GetCriterion(i) < second->GetCriterion(i))
				second_worse = true;
			else if (first->GetCriterion(i) > second->GetCriterion(i))
				first_worse = true;
		}
		if (!first_worse && second_worse)
			return g_domin_first;
		else if (first_worse && !second_worse)
			return g_domin_second;
	}
	return g_domin_none;
}
//---------------------------------------------------------------------------

UINT Probl::GetDominationK(DemoInd *first, DemoInd *second, UINT k)
{
	bool first_worse = false;
	bool second_worse = false;

	for (UINT i = k; i < m_num_func; i++) {
		if (first->GetCriterion(i) < second->GetCriterion(i))
			second_worse = true;
		else if (first->GetCriterion(i) > second->GetCriterion(i))
			first_worse = true;
	}
	if (!first_worse && second_worse)
		return g_domin_first;
	else if (first_worse && !second_worse)
		return g_domin_second;

	return g_domin_none;
}
//---------------------------------------------------------------------------

double Probl::GetDistance(vector<double> &crit1,
													vector<double> &crit2)
{
	double result = 0;

	if (IsEqualCrit(crit1, crit2))
		return result;

	for (UINT i = 0; i < m_num_func; i++)
		result += pow(crit1.at(i) - crit2.at(i), 2);

	return sqrt(result);
}
//---------------------------------------------------------------------------

string Probl::GetVariableString()
{
	stringstream result;

	result.precision(g_out_prec);
	for (UINT i = 0; i < m_num_var; i++) {
		result << "\t" << scientific << m_var_min.at(i);
		result << "\t" << scientific << m_var_max.at(i);
		result << "\t" << scientific << m_var_disc.at(i) << endl;
	}

	return (result.str());
}
//---------------------------------------------------------------------------

string Probl::GetFunctionString()
{
	stringstream result;

	result.precision(g_out_prec);
	for (UINT i = 0; i < m_num_func; i++) {
		result << "\t" << scientific << m_func_lower.at(i);
		result << "\t" << scientific << m_func_upper.at(i) << endl;
	}

	return (result.str());
}
//---------------------------------------------------------------------------

string Probl::GetProblemString()
{
	string result;

	switch (m_function) {
		case g_prob_extern: result = "extern evaluation"; break;
		case g_prob_fun_ZDT1: result = "ZDT1"; break;
		case g_prob_fun_ZDT2: result = "ZDT2"; break;
		case g_prob_fun_ZDT3: result = "ZDT3"; break;
		case g_prob_fun_ZDT4: result = "ZDT4"; break;
		case g_prob_fun_ZDT6: result = "ZDT6"; break;
		case g_prob_fun_WFG1: result = "WFG1"; break;
		case g_prob_fun_WFG2: result = "WFG2"; break;
		case g_prob_fun_WFG3: result = "WFG3"; break;
		case g_prob_fun_WFG4: result = "WFG4"; break;
		case g_prob_fun_WFG5: result = "WFG5"; break;
		case g_prob_fun_WFG6: result = "WFG6"; break;
		case g_prob_fun_WFG7: result = "WFG7"; break;
		case g_prob_fun_WFG8: result = "WFG8"; break;
		case g_prob_fun_WFG9: result = "WFG9"; break;
		case g_prob_fun_DTLZ1: result = "DTLZ1"; break;
		case g_prob_fun_DTLZ2: result = "DTLZ2"; break;
		case g_prob_fun_DTLZ3: result = "DTLZ3"; break;
		case g_prob_fun_DTLZ4: result = "DTLZ4"; break;
		case g_prob_fun_DTLZ5: result = "DTLZ5"; break;
		case g_prob_fun_DTLZ6: result = "DTLZ6"; break;
		case g_prob_fun_DTLZ7: result = "DTLZ7"; break; 
		case g_prob_fun_DTLZ6s: result = "DTLZ6s"; break;
		case g_prob_fun_DTLZ7s: result = "DTLZ7s"; break;
	}
	return result;
}
//---------------------------------------------------------------------------

