/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: probl_fun.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "probl_fun.h"
//---------------------------------------------------------------------------

#include "WFG/exampleproblems.h"
using namespace WFG::Toolkit;
using namespace WFG::Toolkit::Examples;
//---------------------------------------------------------------------------  

const UINT g_max_shift_var = 30;
//---------------------------------------------------------------------------

ProblFun::ProblFun()
	: Probl()
{
	m_function = g_prob_fun_WFG1;
	m_wfg_M = 2;
	m_wfg_k = 6;
	m_wfg_l = 4;

	m_shift_o = new double [g_max_shift_var];
	m_shift_d = new double [g_max_shift_var];
	m_shift_l = new double [g_max_shift_var];

	UINT i = 0;
	m_shift_o[i++] = 0.366;	m_shift_o[i++] = 0.303;	m_shift_o[i++] = 0.852;
	m_shift_o[i++] = 0.759;	m_shift_o[i++] = 0.950;	m_shift_o[i++] = 0.558;
	m_shift_o[i++] = 0.014;	m_shift_o[i++] = 0.596;	m_shift_o[i++] = 0.816;
	m_shift_o[i++] = 0.977;	m_shift_o[i++] = 0.222;	m_shift_o[i++] = 0.704;
	m_shift_o[i++] = 0.522;	m_shift_o[i++] = 0.933;	m_shift_o[i++] = 0.713;
	m_shift_o[i++] = 0.228;	m_shift_o[i++] = 0.450;	m_shift_o[i++] = 0.172;
	m_shift_o[i++] = 0.969;	m_shift_o[i++] = 0.356;	m_shift_o[i++] = 0.049;
	m_shift_o[i++] = 0.755;	m_shift_o[i++] = 0.895;	m_shift_o[i++] = 0.286;
	m_shift_o[i++] = 0.251;	m_shift_o[i++] = 0.933;	m_shift_o[i++] = 0.131;
	m_shift_o[i++] = 0.941;	m_shift_o[i++] = 0.702;	m_shift_o[i] = 0.848;

	i = 0;
	m_shift_d[i++] = 0.155;	m_shift_d[i++] = 0.119;	m_shift_d[i++] = 0.185;
	m_shift_d[i++] = 0.064;	m_shift_d[i++] = 0.07;	m_shift_d[i++] = 0.203;
	m_shift_d[i++] = 0.166;	m_shift_d[i++] = 0.151;	m_shift_d[i++] = 0.219;
	m_shift_d[i++] = 0.083;	m_shift_d[i++] = 0.161;	m_shift_d[i++] = 0.057;
	m_shift_d[i++] = 0.169;	m_shift_d[i++] = 0.072;	m_shift_d[i++] = 0.135;
	m_shift_d[i++] = 0.114;	m_shift_d[i++] = 0.241;	m_shift_d[i++] = 0.129;
	m_shift_d[i++] = 0.23;	m_shift_d[i++] = 0.181;	m_shift_d[i++] = 0.195;
	m_shift_d[i++] = 0.058;	m_shift_d[i++] = 0.152;	m_shift_d[i++] = 0.107;
	m_shift_d[i++] = 0.119;	m_shift_d[i++] = 0.219;	m_shift_d[i++] = 0.203;
	m_shift_d[i++] = 0.212;	m_shift_d[i++] = 0.126;	m_shift_d[i] = 0.238;
                                     
	i = 0;
	m_shift_l[i++] = 3.236;	m_shift_l[i++] = 4.201;	m_shift_l[i++] = 2.701;
	m_shift_l[i++] = 7.775;	m_shift_l[i++] = 7.148;	m_shift_l[i++] = 2.465;
	m_shift_l[i++] = 3.02;	m_shift_l[i++] = 3.322;	m_shift_l[i++] = 2.285;
	m_shift_l[i++] = 6.004; m_shift_l[i++] = 3.106;	m_shift_l[i++] = 8.801;
	m_shift_l[i++] = 2.956;	m_shift_l[i++] = 6.918;	m_shift_l[i++] = 3.708;
	m_shift_l[i++] = 4.403;	m_shift_l[i++] = 2.077;	m_shift_l[i++] = 3.884;
	m_shift_l[i++] = 2.171;	m_shift_l[i++] = 2.76;	m_shift_l[i++] = 2.567;
	m_shift_l[i++] = 8.636;	m_shift_l[i++] = 3.299;	m_shift_l[i++] = 4.666;
	m_shift_l[i++] = 4.208;	m_shift_l[i++] = 2.285;	m_shift_l[i++] = 2.469;
	m_shift_l[i++] = 2.362;	m_shift_l[i++] = 3.963;	m_shift_l[i] = 2.099;
}
//---------------------------------------------------------------------------      

ProblFun::~ProblFun()
{
	delete[] m_shift_o;
	delete[] m_shift_d;
	delete[] m_shift_l;
}
//---------------------------------------------------------------------------

void ProblFun::InitProblem()
{
	m_var_min.clear();
	m_var_max.clear();
	m_var_disc.clear();

	m_func_lower.clear();
	m_func_upper.clear();

	if ((m_function == g_prob_fun_ZDT1) ||
			(m_function == g_prob_fun_ZDT2) ||
			(m_function == g_prob_fun_ZDT3)) {
		m_num_var = 30;
		m_num_func = 2;
		m_num_feat = 0;
		for (UINT i = 0; i < m_num_var; i++) {
			m_var_min.push_back(0);
			m_var_max.push_back(1);
			m_var_disc.push_back(0);
		}
		m_func_lower.push_back(0);
		m_func_upper.push_back(1);
		m_func_lower.push_back(0);
		m_func_upper.push_back(10);
	}
	else if (m_function == g_prob_fun_ZDT4) {
		m_num_var = 10;
		m_num_func = 2;
		m_num_feat = 0;

		m_var_min.push_back(0);
		m_var_max.push_back(1);
		m_var_disc.push_back(0);
		for (UINT i = 1; i < m_num_var; i++) {
			m_var_min.push_back(-5);
			m_var_max.push_back(5);
			m_var_disc.push_back(0);
		}
		m_func_lower.push_back(0);
		m_func_upper.push_back(1);
		m_func_lower.push_back(0);
		m_func_upper.push_back(500);
	}
	else if (m_function == g_prob_fun_ZDT6) {
		m_num_var = 10;
		m_num_func = 2;
		m_num_feat = 0;
		for (UINT i = 0; i < m_num_var; i++) {
			m_var_min.push_back(0);
			m_var_max.push_back(1);
			m_var_disc.push_back(0);
		}
		m_func_lower.push_back(0);
		m_func_upper.push_back(1);
		m_func_lower.push_back(0);
		m_func_upper.push_back(20);
	}
	else if ((m_function == g_prob_fun_WFG1) ||
					 (m_function == g_prob_fun_WFG2) ||
					 (m_function == g_prob_fun_WFG3) ||
					 (m_function == g_prob_fun_WFG4) ||
					 (m_function == g_prob_fun_WFG5) ||
					 (m_function == g_prob_fun_WFG6) ||
					 (m_function == g_prob_fun_WFG7) ||
					 (m_function == g_prob_fun_WFG8) ||
					 (m_function == g_prob_fun_WFG9)) {
		m_num_var = m_wfg_k + m_wfg_l;
		m_num_func = m_wfg_M;
		m_num_feat = 0;
		for (UINT i = 0; i < m_num_var; i++) {
			m_var_min.push_back(0);
			m_var_max.push_back(2*(i+1));
			m_var_disc.push_back(0);
		}
		for (UINT i = 0; i < m_num_func; i++) {
			m_func_lower.push_back(0);
			m_func_upper.push_back(3 * (m_num_func + 1));
		}
	}
	else if ((m_function == g_prob_fun_DTLZ1) ||
					 (m_function == g_prob_fun_DTLZ2) ||
					 (m_function == g_prob_fun_DTLZ3) ||
					 (m_function == g_prob_fun_DTLZ4) ||
					 (m_function == g_prob_fun_DTLZ5) ||
					 (m_function == g_prob_fun_DTLZ6) ||
					 (m_function == g_prob_fun_DTLZ7)) {

		m_num_func = m_wfg_M;

		if (m_function == g_prob_fun_DTLZ1) {
			m_num_var = 5 + m_wfg_M - 1;
		}
		else if (m_function == g_prob_fun_DTLZ7) {
			m_num_var = 20 + m_wfg_M - 1;
		}
		else {
			m_num_var = 10 + m_wfg_M - 1;
		}
		m_num_feat = 0;
		for (UINT i = 0; i < m_num_var; i++) {
			m_var_min.push_back(0);
			m_var_max.push_back(1);
			m_var_disc.push_back(0);
		}
		for (UINT i = 0; i < m_num_func; i++) {
			m_func_lower.push_back(0);
			m_func_upper.push_back(10);
		}
	}                  
	else if ((m_function == g_prob_fun_DTLZ6s) ||
					 (m_function == g_prob_fun_DTLZ7s)) {

		m_num_func = m_wfg_M;

		if (m_function == g_prob_fun_DTLZ6s) {
			m_num_var = 10 + m_wfg_M - 1;
		}
		else if (m_function == g_prob_fun_DTLZ7s) {
			m_num_var = 20 + m_wfg_M - 1;
    }

		m_num_feat = 0;
		for (UINT i = 0; i < m_num_var; i++) {             
			m_var_min.push_back(0);
			m_var_max.push_back(1);
			m_var_disc.push_back(0);
		}
		for (UINT i = 0; i < m_num_func; i++) {
			m_func_lower.push_back(1);
			m_func_upper.push_back(11);
		}
	}
}
//---------------------------------------------------------------------------

void ProblFun::Evaluate(DemoInd *ind)
{
	if (GetNumEval() % 100 == 0) {
		cout << "evaluating individual " << GetNumEval() << endl;
	}

	if (m_function == g_prob_fun_ZDT1) CalcZDT1(ind);
	else if (m_function == g_prob_fun_ZDT2) CalcZDT2(ind);
	else if (m_function == g_prob_fun_ZDT3) CalcZDT3(ind);
	else if (m_function == g_prob_fun_ZDT4) CalcZDT4(ind);
	else if (m_function == g_prob_fun_ZDT6) CalcZDT6(ind);
	else if (m_function == g_prob_fun_WFG1) CalcWFG1(ind);
	else if (m_function == g_prob_fun_WFG2) CalcWFG2(ind);
	else if (m_function == g_prob_fun_WFG3) CalcWFG3(ind);
	else if (m_function == g_prob_fun_WFG4) CalcWFG4(ind);
	else if (m_function == g_prob_fun_WFG5) CalcWFG5(ind);
	else if (m_function == g_prob_fun_WFG6) CalcWFG6(ind);
	else if (m_function == g_prob_fun_WFG7) CalcWFG7(ind);
	else if (m_function == g_prob_fun_WFG8) CalcWFG8(ind);
	else if (m_function == g_prob_fun_WFG9) CalcWFG9(ind);
	else if (m_function == g_prob_fun_DTLZ1) CalcDTLZ1(ind);
	else if (m_function == g_prob_fun_DTLZ2) CalcDTLZ2(ind);
	else if (m_function == g_prob_fun_DTLZ3) CalcDTLZ3(ind);
	else if (m_function == g_prob_fun_DTLZ4) CalcDTLZ4(ind);
	else if (m_function == g_prob_fun_DTLZ5) CalcDTLZ5(ind);
	else if (m_function == g_prob_fun_DTLZ6) CalcDTLZ6(ind);
	else if (m_function == g_prob_fun_DTLZ7) CalcDTLZ7(ind); 
	else if (m_function == g_prob_fun_DTLZ6s) CalcDTLZ6s(ind);
	else if (m_function == g_prob_fun_DTLZ7s) CalcDTLZ7s(ind);
}
//---------------------------------------------------------------------------

void ProblFun::CalcZDT1(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	result.push_back(ind->GetGene(0));
	double sum = 1;
	for (UINT i = 1; i < m_num_var; i++)
		sum += 9 * ind->GetGene(i) / (m_num_var - 1);
	result.push_back(sum * (1 - sqrt(ind->GetGene(0) / sum)));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcZDT2(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	result.push_back(ind->GetGene(0));
	double sum = 1;
	for (UINT i = 1; i < m_num_var; i++)
		sum += 9 * ind->GetGene(i) / (m_num_var - 1);
	result.push_back(sum * (1 - pow(ind->GetGene(0) / sum, 2)));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcZDT3(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	result.push_back(ind->GetGene(0));
	double sum = 1;
	for (UINT i = 1; i < m_num_var; i++)
		sum += 9 * ind->GetGene(i) / (m_num_var - 1);
	result.push_back(sum * (1 - sqrt(ind->GetGene(0) / sum) -
		(ind->GetGene(0) / sum) * sin(10 * M_PI * ind->GetGene(0))));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcZDT4(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	result.push_back(ind->GetGene(0));
	double sum = 91;
	for (UINT i = 1; i < m_num_var; i++)
		sum += pow(ind->GetGene(i), 2) - 10 * cos(4 * M_PI * ind->GetGene(i));
	result.push_back(sum * (1 - sqrt(ind->GetGene(0) / sum)));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcZDT6(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	result.push_back(1 - exp(-4 * ind->GetGene(0)) *
		pow(sin(6 * M_PI * ind->GetGene(0)), 6));
	double sum = 0;
	for (UINT i = 1; i < m_num_var; i++)
		sum += ind->GetGene(i) / 9;
	sum = 1 + 9 * pow(sum, 0.25);
	result.push_back(sum * (1 - pow(result.at(0) / sum, 2)));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG1(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG1(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG2(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG2(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG3(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG3(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG4(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG4(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG5(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG5(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG6(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG6(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG7(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG7(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG8(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG8(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcWFG9(DemoInd *ind)
{
	ind->SetCriteria(Problems::WFG9(ind->GetGenes(), m_wfg_k, m_wfg_M));
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ1(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;

	double g = 0;
	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(ind->GetGene(i - 1) - 0.5, 2) -
			cos(20 * M_PI * (ind->GetGene(i - 1) - 0.5));
	}
	g = 100 * (k + g);

	for (i = 1; i <= m_num_func; i++) {
		double f = 0.5 * (1 + g);
		for (j = m_num_func - i; j >= 1; j--) {
			f *= ind->GetGene(j - 1);
		}
		if (i > 1) {
			f *= 1 - ind->GetGene((m_num_func - i + 1) - 1);
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ2(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;

	double g = 0;
	for (i = n - k + 1; i <= n; i++) {
		g += pow(ind->GetGene(i - 1) - 0.5, 2);
	}

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(ind->GetGene(j - 1) * M_PI / 2);
		}
		if (i > 1) {
			f *= sin(ind->GetGene((m_num_func - i + 1) - 1) * M_PI / 2);
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ3(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;

	double g = 0;
	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(ind->GetGene(i - 1) - 0.5, 2) -
			cos(20 * M_PI * (ind->GetGene(i - 1) - 0.5));
	}
	g = 100 * (k + g);

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(ind->GetGene(j - 1) * M_PI / 2);
		}
		if (i > 1) {
			f *= sin(ind->GetGene((m_num_func - i + 1) - 1) * M_PI / 2);
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ4(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;
	double alpha = 100;

	double g = 0;

	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(ind->GetGene(i - 1) - 0.5, 2);
	}

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(pow(ind->GetGene(j - 1), alpha) * M_PI / 2);
		}
		if (i > 1) {
			f *= sin(pow(ind->GetGene((m_num_func - i + 1) - 1), alpha) * M_PI / 2);
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ5(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;
	vector<double> theta;
	double t;
	double g = 0;

	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(ind->GetGene(i - 1) - 0.5, 2);
	}

	t = M_PI / (4 * (1 + g));
	theta.push_back(ind->GetGene(0) * M_PI / 2);
	for (i = 2; i <= m_num_func - 1; i++) {
		theta.push_back(t * (1 + 2 * g * ind->GetGene(i - 1)));
	}

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(theta.at(j-1));
		}
		if (i > 1) {
			f *= sin(theta.at((m_num_func - i + 1) - 1));
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ6(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;
	vector<double> theta;
	double t;
	double g = 0;

	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(ind->GetGene(i - 1), 0.1);
	}

	t = M_PI / (4 * (1 + g));
	theta.push_back(ind->GetGene(0) * M_PI / 2);
	for (i = 2; i <= m_num_func - 1; i++) {
		theta.push_back(t * (1 + 2 * g * ind->GetGene(i - 1)));
	}

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(theta.at(j-1));
		}
		if (i > 1) {
			f *= sin(theta.at((m_num_func - i + 1) - 1));
		}
		result.push_back(f);
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------

void ProblFun::CalcDTLZ7(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;
	double g = 0;
	double h = 0;

	for (i = n - k + 1; i <= n; i++) {
		g += ind->GetGene(i - 1);
	}
	g = 1 + 9 * g / k;

	for (i = 1; i <= m_num_func - 1; i++) {
		result.push_back(ind->GetGene(i - 1));
	}

	for (j = 1; j <= m_num_func - 1; j++) {
		h += ind->GetGene(j - 1) / (1 + g) * (1 + sin(3 * M_PI * ind->GetGene(j - 1)));
	}
	h = m_num_func - h;
	result.push_back((1 + g) * h);

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------     

void ProblFun::CalcDTLZ6s(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = n - m_num_func + 1;
	vector<double> theta;
	double t;
	double g = 0;

	double y;
	vector<double> z, p, p_sum;

	for (i = 0; i < n; i++) {
		y = ((1 + m_shift_d[i]) * ind->GetGene(i)) + (m_shift_o[i] - m_shift_d[i]);
		if (y >= m_shift_o[i]) {
			z.push_back(y - m_shift_o[i]);
			p.push_back(0);
		}
		else {
			z.push_back(m_shift_l[i] * (m_shift_o[i] - y));
			p.push_back((m_shift_o[i] - y) / m_shift_d[i]);
		}
	}
	
	for (j = 0; j < m_num_func; j++) {
		p_sum.push_back(0);
	}

	for (i = n - k + 1; i <= n; i++) 	{
		g += pow(z.at(i - 1), 0.1);
		for (j = 0; j < m_num_func; j++) {
			p_sum.at(j) += pow(p.at(i - 1), 2);
		}
	}

	t = M_PI / (4 * (1 + g));
	theta.push_back(z.at(0) * M_PI / 2);
	for (i = 2; i <= m_num_func - 1; i++) {
		theta.push_back(t * (1 + 2 * g * z.at(i - 1)));
	}

	for (i = 1; i <= m_num_func; i++) {
		double f = 1 + g;
		for (j = m_num_func - i; j >= 1; j--) {
			f *= cos(theta.at(j - 1));
			p_sum.at(i - 1) += pow(p.at(j - 1), 2);
		}
		if (i > 1) {
			f *= sin(theta.at((m_num_func - i + 1) - 1));
			p_sum.at(i - 1) += pow(p.at((m_num_func - i + 1) - 1), 2);
		}
		result.push_back(2.0 / (1 + exp(-sqrt(p_sum.at(i - 1)))) * (f + 1));
	}

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------   

void ProblFun::CalcDTLZ7s(DemoInd *ind)
{
	vector<double> result;
	result.clear();

	UINT i, j;
	UINT n = m_num_var;
	UINT k = m_num_var - m_num_func + 1;
	double g = 0;
	double h = 0;

	double f;      
	vector<double> z, p;
	double p_sum_M;
	double y;

	for (i = 0; i < m_num_var; i++) {
		y = ((1 + m_shift_d[i]) * ind->GetGene(i)) + (m_shift_o[i] - m_shift_d[i]);
		if (y >= m_shift_o[i]) {
			z.push_back(y - m_shift_o[i]);
			p.push_back(0);
		}
		else {
			z.push_back(m_shift_l[i] * (m_shift_o[i] - y));
			p.push_back((m_shift_o[i] - y) / m_shift_d[i]);
		}
	}

	for (i = n - k + 1; i <= n; i++) {
		g += z.at(i - 1);
	}
	g = 1 + 9 * g / k;

	for (i = 1; i <= m_num_func - 1; i++) {
		result.push_back(2.0 / (1 + exp(-p.at(i - 1))) * (z.at(i - 1) + 1));
	}

	for (j = 1; j <= m_num_func - 1; j++) {
		h += z.at(j - 1) / (1 + g) * (1 + sin(3 * M_PI * z.at(j - 1)));
	}
	h = m_num_func - h;
	f = (1 + g) * h;

	p_sum_M = 0;                      
	for (i = n - k + 1; i <= n; i++)
		p_sum_M += pow(p.at(i - 1), 2);

	result.push_back(2.0 / (1 + exp(-sqrt(p_sum_M))) * (f + 1));

	ind->SetCriteria(result);
	ind->SetEvaluated(true);
	ind->SetViolation(0);
}
//---------------------------------------------------------------------------


