/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class ProblFun, which is derived from class Probl.

	file: probl_fun.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef probl_funH
#define probl_funH
//---------------------------------------------------------------------------

#include "demo_params.h"
#include "probl.h"
//--------------------------------------------------------------------------- 

/** Maximum nuber of possible variables for the shifted problem functions. */
extern const UINT g_max_shift_var;
//---------------------------------------------------------------------------

class ProblFun : public Probl
{
	/** Parameter M (dimensionality of the objective space) in the WFG and
	DTLZ problems. */
	UINT m_wfg_M;

	/** Parameter k in the WFG problems. */
	UINT m_wfg_k;

	/** Parameter l in the WFG problems. */
	UINT m_wfg_l;
                             
	/** Parameters o, d and lambda for the shifted problem funcitons (same as the
	parameters of the shifted DTLZ2 function from CEC 2007 competition). */
	double *m_shift_o;
	double *m_shift_d;
	double *m_shift_l;

public:
	inline void SetWfgM(UINT wfg_M) {m_wfg_M = wfg_M;};
	inline void SetWfgK(UINT wfg_k) {m_wfg_k = wfg_k;};
	inline void SetWfgL(UINT wfg_l) {m_wfg_l = wfg_l;};

	inline UINT GetWfgM() {return m_wfg_M;};
	inline UINT GetWfgK() {return m_wfg_k;};
	inline UINT GetWfgL() {return m_wfg_l;};

	/** Class constructor. */
	ProblFun();

	/** Class destructor. */
	~ProblFun();

	/** Initialization function sets the parameters of the problem. */
	void InitProblem();

	/** Function calls the evaluation of the current individual.
	params: ind the current individual
	*/
	void Evaluate(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the ZDT1
	problem.
	params: ind the current individual
	*/
	void CalcZDT1(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the ZDT2
	problem.
	params: ind the current individual
	*/
	void CalcZDT2(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the ZDT3
	problem.
	params: ind the current individual
	*/
	void CalcZDT3(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the ZDT4
	problem.
	params: ind the current individual
	*/
	void CalcZDT4(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the ZDT6
	problem.
	params: ind the current individual
	*/
	void CalcZDT6(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG1
	problem.
	params: ind the current individual
	*/
	void CalcWFG1(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG2
	problem.
	params: ind the current individual
	*/
	void CalcWFG2(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG3
	problem.
	params: ind the current individual
	*/
	void CalcWFG3(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG4
	problem.
	params: ind the current individual
	*/
	void CalcWFG4(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG5
	problem.
	params: ind the current individual
	*/
	void CalcWFG5(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG6
	problem.
	params: ind the current individual
	*/
	void CalcWFG6(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG7
	problem.
	params: ind the current individual
	*/
	void CalcWFG7(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG8
	problem.
	params: ind the current individual
	*/
	void CalcWFG8(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the WFG9
	problem.
	params: ind the current individual
	*/
	void CalcWFG9(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ1
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ1(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ2
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ2(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ3
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ3(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ4
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ4(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ5
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ5(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ6
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ6(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the DTLZ7
	problem.
	params: ind the current individual
	*/
	void CalcDTLZ7(DemoInd *ind);     

	/** Function calculates the individual's criteria and violations for the
	shifted DTLZ6 problem.
	params: ind the current individual
	*/
	void CalcDTLZ6s(DemoInd *ind);

	/** Function calculates the individual's criteria and violations for the 
	shifted DTLZ7 problem.
	params: ind the current individual
	*/
	void CalcDTLZ7s(DemoInd *ind);
};
//---------------------------------------------------------------------------
#endif

